@@ -4,16 +4,16 @@
  *
  * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
  *
- * Licensed under the GPL version 2, see the file LICENSE in this tarball.
+ * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 
 #include "libbb.h"
 
 enum {
-	USE_FEATURE_PIDOF_SINGLE(OPTBIT_SINGLE,)
-	USE_FEATURE_PIDOF_OMIT(  OPTBIT_OMIT  ,)
-	OPT_SINGLE = USE_FEATURE_PIDOF_SINGLE((1<<OPTBIT_SINGLE)) + 0,
-	OPT_OMIT   = USE_FEATURE_PIDOF_OMIT(  (1<<OPTBIT_OMIT  )) + 0,
+	IF_FEATURE_PIDOF_SINGLE(OPTBIT_SINGLE,)
+	IF_FEATURE_PIDOF_OMIT(  OPTBIT_OMIT  ,)
+	OPT_SINGLE = IF_FEATURE_PIDOF_SINGLE((1<<OPTBIT_SINGLE)) + 0,
+	OPT_OMIT   = IF_FEATURE_PIDOF_OMIT(  (1<<OPTBIT_OMIT  )) + 0,
 };
 
 int pidof_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
@@ -28,19 +28,19 @@ int pidof_main(int argc UNUSED_PARAM, char **argv)
 
 	/* do unconditional option parsing */
 	opt = getopt32(argv, ""
-			USE_FEATURE_PIDOF_SINGLE ("s")
-			USE_FEATURE_PIDOF_OMIT("o:", &omits));
+			IF_FEATURE_PIDOF_SINGLE ("s")
+			IF_FEATURE_PIDOF_OMIT("o:", &omits));
 
 #if ENABLE_FEATURE_PIDOF_OMIT
 	/* fill omit list.  */
 	{
 		llist_t *omits_p = omits;
-		while (omits_p) {
+		while (1) {
+			omits_p = llist_find_str(omits_p, "%PPID");
+			if (!omits_p)
+				break;
 			/* are we asked to exclude the parent's process ID?  */
-			if (strcmp(omits_p->data, "%PPID") == 0) {
-				omits_p->data = utoa((unsigned)getppid());
-			}
-			omits_p = omits_p->link;
+			omits_p->data = utoa((unsigned)getppid());
 		}
 	}
 #endif
